@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
-## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import cups
+cups.require ("1.9.42")
 import sys
 import statereason
 from statereason import StateReason
@@ -45,10 +46,12 @@ except locale.Error, e:
 
 APPDIR="/usr/share/system-config-printer"
 DOMAIN="system-config-printer"
-GLADE="applet.glade"
 ICON="printer"
 SEARCHING_ICON="document-print-preview"
 
+# We need to call pynotify.init before we can check the server for caps
+pynotify.init('System Config Printer Notification')
+
 ####
 #### NewPrinterNotification DBus server (the 'new' way).
 ####
@@ -61,8 +64,6 @@ class NewPrinterNotification(dbus.service.Object):
     STATUS_GENERIC_DRIVER = 2
     STATUS_NO_DRIVER = 3
 
-    INSTALL_PACKAGES_COMMAND="/usr/bin/system-install-packages"
-
     def __init__ (self, bus):
         self.bus = bus
         self.getting_ready = 0
@@ -103,67 +104,123 @@ class NewPrinterNotification(dbus.service.Object):
 
     @dbus.service.method(PDS_IFACE, in_signature='isssss', out_signature='')
     def NewPrinter (self, status, name, mfg, mdl, des, cmd):
-        #status = 1
         global viewer
         self.wake_up ()
-        c = cups.Connection ()
-        try:
-            printer = c.getPrinters ()[name]
-        except KeyError:
-            return
 
-        try:
-            filename = c.getPPD (name)
-        except cups.IPPError:
-            return
-
-        del c
-
-        # Check for missing packages
-        ppd = cups.PPD (filename)
-        import os
-        os.unlink (filename)
-        import sys
-        sys.path.append (APPDIR)
-        import cupshelpers
-        (missing_pkgs,
-         missing_exes) = cupshelpers.missingPackagesAndExecutables (ppd)
-
-        from cupshelpers.ppds import ppdMakeModelSplit
-        (make, model) = ppdMakeModelSplit (printer['printer-make-and-model'])
-        driver = make + " " + model
-        if status < self.STATUS_GENERIC_DRIVER:
-            title = _("Printer added")
-        else:
+        if name.find("/") >= 0:
+            # name is a URI, no queue was generated, because no suitable
+            # driver was found
             title = _("Missing printer driver")
-
-        if len (missing_pkgs) > 0:
-            pkgs = reduce (lambda x,y: x + ", " + y, missing_pkgs)
-            title = _("Install printer driver")
-            text = _("`%s' requires driver installation: %s.") % (name, pkgs)
-            n = pynotify.Notification (title, text)
-            n.set_urgency (pynotify.URGENCY_CRITICAL)
-            if os.access (self.INSTALL_PACKAGES_COMMAND, os.X_OK):
-                n.add_action ("install-driver", _("Install"),
-                              lambda x, y: self.install_driver (x, y,
-                                                                missing_pkgs))
-        elif status == self.STATUS_SUCCESS:
-            text = _("`%s' is ready for printing.") % name
-            n = pynotify.Notification (title, text)
-            n.set_urgency (pynotify.URGENCY_NORMAL)
-            n.add_action ("configure", _("Configure"),
-                          lambda x, y: self.configure (x, y, name))
-        else: # Model mismatch
             devid = "MFG:%s;MDL:%s;DES:%s;CMD:%s;" % \
                 (mfg, mdl, des, cmd)
-            text = _("`%s' has been added, using the `%s' driver.") % \
-                   (name, driver)
+            if (mfg and mdl) or des:
+                if (mfg and mdl):
+                    device = "%s %s" % (mfg, mdl)
+                else:
+                    device = des
+                text = _("No printer driver for %s.") % device
+            else:
+                text = _("No driver for this printer.")
             n = pynotify.Notification (title, text, 'printer')
-            n.set_urgency (pynotify.URGENCY_CRITICAL)
-            n.add_action ("find-driver", _("Find driver"),
-                          lambda x, y: self.find_driver (x, y, name, devid))
+            if "actions" in pynotify.get_server_caps():
+                n.set_urgency (pynotify.URGENCY_CRITICAL)
+                n.add_action ("setup-printer", _("Search"),
+                              lambda x, y:
+                                  self.setup_printer (x, y, name, devid))
+            else:
+                args = ["--setup-printer", name]
+                if devid != "": args = args + ["--devid", devid]
+                self.run_config_tool (args)
+
+        else:
+            # name is the name of the queue which hal_lpadmin has set up
+            # automatically.
+            c = cups.Connection ()
+            try:
+                printer = c.getPrinters ()[name]
+            except KeyError:
+                return
+
+            try:
+                filename = c.getPPD (name)
+            except cups.IPPError:
+                return
+
+            del c
+
+            # Check for missing packages
+            ppd = cups.PPD (filename)
+            import os
+            os.unlink (filename)
+            import sys
+            sys.path.append (APPDIR)
+            import cupshelpers
+            (missing_pkgs,
+             missing_exes) = cupshelpers.missingPackagesAndExecutables (ppd)
+
+            from cupshelpers.ppds import ppdMakeModelSplit
+            (make, model) = ppdMakeModelSplit (printer['printer-make-and-model'])
+            driver = make + " " + model
+            if status < self.STATUS_GENERIC_DRIVER:
+                title = _("Printer added")
+            else:
+                title = _("Missing printer driver")
+
+            if len (missing_pkgs) > 0:
+                pkgs = reduce (lambda x,y: x + ", " + y, missing_pkgs)
+                title = _("Install printer driver")
+                text = _("`%s' requires driver installation: %s.") % (name, pkgs)
+                n = pynotify.Notification (title, text)
+                import installpackage
+                if "actions" in pynotify.get_server_caps():
+                    try:
+                        self.packagekit = installpackage.PackageKit ()
+                        n.add_action ("install-driver", _("Install"),
+                                      lambda x, y:
+                                          self.install_driver (x, y,
+                                                               missing_pkgs))
+                    except:
+                        pass
+                else:
+                    try:
+                        self.packagekit = installpackage.PackageKit ()
+                        self.packagekit.InstallPackageName (0, 0,
+                                                            missing_pkgs[0])
+                    except:
+                        pass
+
+            elif status == self.STATUS_SUCCESS:
+                text = _("`%s' is ready for printing.") % name
+                n = pynotify.Notification (title, text)
+                if "actions" in pynotify.get_server_caps():
+                    n.set_urgency (pynotify.URGENCY_NORMAL)
+                    n.add_action ("test-page", _("Print test page"),
+                                  lambda x, y:
+                                      self.print_test_page (x, y, name, devid))
+                    n.add_action ("configure", _("Configure"),
+                                  lambda x, y: self.configure (x, y, name))
+                else:
+                    self.run_config_tool (["--configure-printer",
+                                           name, "--no-focus-on-map"])
+            else: # Model mismatch
+                devid = "MFG:%s;MDL:%s;DES:%s;CMD:%s;" % \
+                    (mfg, mdl, des, cmd)
+                text = _("`%s' has been added, using the `%s' driver.") % \
+                    (name, driver)
+                n = pynotify.Notification (title, text, 'printer')
+                if "actions" in pynotify.get_server_caps():
+                    n.set_urgency (pynotify.URGENCY_CRITICAL)
+                    n.add_action ("test-page", _("Print test page"),
+                                  lambda x, y:
+                                      self.print_test_page (x, y, name, devid))
+                    n.add_action ("find-driver", _("Find driver"),
+                                  lambda x, y: 
+                                  self.find_driver (x, y, name, devid))
+                    n.set_timeout (pynotify.EXPIRES_NEVER)
+                else:
+                    self.run_config_tool (["--configure-printer",
+                                           name, "--no-focus-on-map"])
 
-        n.set_timeout (pynotify.EXPIRES_NEVER)
         viewer.notify_new_printer (name, n)
         # Set the icon back how it was.
         self.timeout_ready ()
@@ -181,7 +238,13 @@ class NewPrinterNotification(dbus.service.Object):
             print "Error forking process"
         else:
             gobject.timeout_add (60 * 1000, self.collect_exit_code, pid)
-        
+
+    def print_test_page (self, notification, action, name, devid = ""):
+        args = ["--print-test-page", name]
+        if devid != "":
+            args.extend (["--devid", devid])
+        self.run_config_tool (args)
+
     def configure (self, notification, action, name):
         self.run_config_tool (["--configure-printer", name])
 
@@ -190,19 +253,16 @@ class NewPrinterNotification(dbus.service.Object):
         if devid != "": args = args + ["--devid", devid]
         self.run_config_tool (args)
 
+    def setup_printer (self, notification, action, uri, devid = ""):
+        args = ["--setup-printer", uri]
+        if devid != "": args = args + ["--devid", devid]
+        self.run_config_tool (args)
+
     def install_driver (self, notification, action, missing_pkgs):
-        import os
-        pid = os.fork ()
-        if pid == 0:
-            # Child.
-            argv = [self.INSTALL_PACKAGES_COMMAND]
-            argv.extend (missing_pkgs)
-            os.execv (argv[0], argv)
-            sys.exit (1)
-        elif pid == -1:
-            print "Error forking process"
-        else:
-            gobject.timeout_add (60 * 1000, self.collect_exit_code, pid)
+        try:
+            self.packagekit.InstallPackageName (0, 0, missing_pkgs[0])
+        except:
+            pass
 
     def collect_exit_code (self, pid):
         # We do this with timers instead of signals because we already
@@ -280,16 +340,20 @@ if __name__ == '__main__':
             bus = dbus.SessionBus()
             bus.add_signal_receiver (monitor_session)
         except:
-            print >> sys.stderr, "%s: failed to connect to session D-Bus" % \
-                PROGRAM_NAME
-            sys.exit (1)
+            try:
+                print >> sys.stderr, ("%s: failed to connect to "
+                                      "session D-Bus" % PROGRAM_NAME)
+            finally:
+                sys.exit (1)
 
     try:
         bus = dbus.SystemBus()
     except:
-        print >> sys.stderr, ("%s: failed to connect to system D-Bus" %
-                              PROGRAM_NAME)
-        sys.exit (1)
+        try:
+            print >> sys.stderr, ("%s: failed to connect to system D-Bus" %
+                                  PROGRAM_NAME)
+        finally:
+            sys.exit (1)
 
     if trayicon:
         try:
@@ -308,12 +372,7 @@ if __name__ == '__main__':
         def any_jobs ():
             try:
                 c = cups.Connection ()
-                try:
-                    jobs = c.getJobs (my_jobs=True, limit=1)
-                except TypeError:
-                    # limit parameter requires pycups >= 1.9.42.
-                    jobs = c.getJobs (my_jobs=True)
-
+                jobs = c.getJobs (my_jobs=True, limit=1)
                 if len (jobs):
                     return True
             except:
