@@ -1,8 +1,8 @@
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009 Red Hat, Inc.
 ## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
-## Copyright (C) 2006, 2007, 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2006, 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 ## along with this program; if not, write to the Free Software
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-import cups, pprint, os, tempfile, re
+import cups, pprint, os, tempfile, re, string
 import locale
 from . import _debugprint
 
@@ -48,12 +48,19 @@ class Printer:
         return "<cupshelpers.Printer \"%s\">" % self.name
 
     def _expand_flags(self):
+
+        def _ascii_lower(str):
+            return str.translate(string.maketrans(string.ascii_uppercase,
+                                                  string.ascii_lowercase));
+
         prefix = "CUPS_PRINTER_"
         prefix_length = len(prefix)
+
         # loop over cups constants
         for name in cups.__dict__:
             if name.startswith(prefix):
-                attr_name = name[prefix_length:].lower()
+                attr_name = \
+                    _ascii_lower(name[prefix_length:])
                 if attr_name in self._flags_blacklist: continue
                 if attr_name == "class": attr_name = "is_class"
                 # set as attribute
@@ -123,11 +130,6 @@ class Printer:
                     
                 if attrs.has_key(name+"-supported"):
                     supported = attrs[name+"-supported"]
-                    # Work around pycups bug fixed in 1.9.40:
-                    # finishings-supported should be a list.
-                    if (type (supported) != list and
-                        type (supported) != tuple):
-                        supported = [supported]
                     self.possible_attributes[name] = (value, supported)
             elif (not key.endswith ("-supported") and
                   key != 'job-sheets-default' and
@@ -366,26 +368,19 @@ class Printer:
         # Also need to check system-wide lpoptions because that's how
         # previous Fedora versions set the default (bug #217395).
         (tmpfd, tmpfname) = tempfile.mkstemp ()
+        os.remove (tmpfname)
         try:
             resource = "/admin/conf/lpoptions"
-
-            try:
-                # Specifying the fd is allowed in pycups >= 1.9.38
-                self.connection.getFile(resource, fd=tmpfd)
-            except TypeError:
-                self.connection.getFile(resource, tmpfname)
+            self.connection.getFile(resource, fd=tmpfd)
         except cups.HTTPError, (s,):
-            try:
-                os.remove (tmpfname)
-            except OSError:
-                pass
-
             if s == cups.HTTP_NOT_FOUND:
                 return False
 
             raise cups.HTTPError (s)
 
-        lines = file (tmpfname).readlines ()
+        f = os.fdopen (tmpfd, 'r+')
+        f.seek (0)
+        lines = f.readlines ()
         changed = False
         i = 0
         for line in lines:
@@ -399,18 +394,15 @@ class Printer:
                 i += 1
 
         if changed:
-            file (tmpfname, 'w').writelines (lines)
+            f.seek (0)
+            f.writelines (lines)
+            f.truncate ()
+            os.lseek (tmpfd, 0, os.SEEK_SET)
             try:
-                self.connection.putFile (resource, tmpfname)
+                self.connection.putFile (resource, fd=tmpfd)
             except cups.HTTPError, (s,):
-                os.remove (tmpfname)
                 return False
 
-        try:
-            os.remove (tmpfname)
-        except OSError:
-            pass
-
         return changed
 
 def getPrinters(connection):
@@ -423,34 +415,12 @@ def getPrinters(connection):
     """
     printers = connection.getPrinters()
     classes = connection.getClasses()
-    printers_conf = None
     for name, printer in printers.iteritems():
         printer = Printer(name, connection, **printer)
         printers[name] = printer
         if classes.has_key(name):
             printer.class_members = classes[name]
             printer.class_members.sort()
-
-        if printer.device_uri.startswith ("smb:"):
-            # smb: URIs may have been sanitized (authentication details
-            # removed), so fetch the actual details from printers.conf.
-            if not printers_conf:
-                printers_conf = _PrintersConf(connection)
-            if printers_conf.device_uris.has_key(name):
-                printer.device_uri = printers_conf.device_uris[name]
-        if not printer.__dict__.has_key ('discovered'):
-            # The CUPS_PRINTER_DISCOVERED flag is new in pycups-1.9.37.
-            printer.discovered = False
-            if printer.device_uri.startswith ("ipp:"):
-                # ipp: Queues can be automatically created as a reaction
-                # to a queue broadcasted by a remote CUPS server. These
-                # queues are not listed in printers,conf. Mark them so
-                # that we can decide which queue entries in the main window
-                # should be editable/deletable and which not
-                if not printers_conf:
-                    printers_conf = _PrintersConf(connection)
-                if not printers_conf.device_uris.has_key(name):
-                    printer.discovered = True
     return printers
 
 def parseDeviceID (id):
@@ -512,6 +482,9 @@ class Device:
         """
         Compare devices by order of preference.
         """
+        if other == None:
+            return -1
+
         if self.is_class != other.is_class:
             if other.is_class:
                 return -1
@@ -541,60 +514,6 @@ class Device:
         
         return result
 
-class _PrintersConf:
-    def __init__(self, connection):
-        """
-        @param connection: CUPS connection
-        @type connection: CUPS.Connection object
-        """
-        self.device_uris = {}
-        self.connection = connection
-        self.parse(self.fetch('/admin/conf/printers.conf'))
-
-    def fetch(self, file):
-        fd, filename = tempfile.mkstemp("printer.conf")
-        try:
-            try:
-                # Specifying the fd is allowed in pycups >= 1.9.38
-                self.connection.getFile(file, fd=fd)
-            except TypeError:
-                self.connection.getFile(file, filename)
-        except cups.HTTPError, e:
-            os.close(fd)
-            try:
-                os.unlink(filename)
-            except OSError:
-                pass
-
-            if (e.args[0] == cups.HTTP_UNAUTHORIZED or
-                e.args[0] == cups.HTTP_NOT_FOUND):
-                return []
-            else:
-                raise e
-
-        os.close(fd)
-        lines = open(filename).readlines()
-        os.unlink(filename)
-        return lines
-
-    def parse(self, lines):
-        current_printer = None
-        for line in lines:
-            words = line.split()
-            if len(words) == 0:
-                continue
-            if words[0] == "DeviceURI":
-                if len (words) >= 2:
-                    self.device_uris[current_printer] = words[1]
-                else:
-                    self.device_uris[current_printer] = ''
-            else:
-                match = re.match(r"<(Default)?Printer ([^>]+)>\s*\n", line) 
-                if match:
-                    current_printer = match.group(2)
-                if line.strip().find("</Printer>") != -1:
-                    current_printer = None
-
 def getDevices(connection):
     """
     Obtain a list of available CUPS devices.
@@ -627,18 +546,7 @@ def activateNewPrinter(connection, name):
     connection.acceptJobs (name)
 
     # Set as the default if there is not already a default printer.
-    default_is_set = False
-    try:
-        if connection.getDefault () != None:
-            default_is_set = True
-    except AttributeError: # getDefault appeared in pycups-1.9.31
-        dests = connection.getDests ()
-        # If a default printer is set it will be available from
-        # key (None,None).
-        if dests.has_key ((None, None)):
-            default_is_set = True
-
-    if not default_is_set:
+    if connection.getDefault () == None:
         connection.setDefault (name)
 
 def copyPPDOptions(ppd1, ppd2):
