@@ -2,9 +2,9 @@
 
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2006, 2007, 2008, 2009 Red Hat, Inc.
 ## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
-## Copyright (C) 2006, 2007, 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2006, 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -43,55 +43,72 @@ def ppdMakeModelSplit (ppd_make_and_model):
     # If the string starts with a known model name (like "LaserJet") assume
     # that the manufacturer name is missing and add the manufacturer name
     # corresponding to the model name
-    if re.search ("^(deskjet|laserjet|designjet|officejet|photosmart|psc|edgeline)", \
-                      ppd_make_and_model, re.I):
+    if re.search ("^\s*(deskjet|dj\b|dj\d|laserjet|lj\b|color\s*laserjet|color\s*lj\b|designjet|officejet|oj\b|photosmart|ps\b|psc)", \
+                      ppd_make_and_model, re.I) or \
+       re.search ("(edgeline)", ppd_make_and_model, re.I):
         make = "HP"
         model = ppd_make_and_model
-    elif re.search ("^(stylus|aculaser)", \
+    elif re.search ("^\s*(stylus|aculaser)", \
                       ppd_make_and_model, re.I):
         make = "Epson"
         model = ppd_make_and_model
-    elif re.search ("^(stylewriter|imagewriter|deskwriter|laserwriter)", \
+    elif re.search ("^\s*(stylewriter|imagewriter|deskwriter|laserwriter)", \
                       ppd_make_and_model, re.I):
         make = "Apple"
         model = ppd_make_and_model
-    elif re.search ("^(pixus|pixma|selphy|imagerunner|\bbjc\b|\bbj\b|\blbp\b)",\
+    elif re.search ("^\s*(pixus|pixma|selphy|imagerunner|\bbjc\b|\bbj\b|\blbp\b)",\
                       ppd_make_and_model, re.I):
         make = "Canon"
         model = ppd_make_and_model
-    elif re.search ("^(\bhl\b|\bdcp\b|\bmfc\b)", \
+    elif re.search ("^\s*(\bhl\b|\bdcp\b|\bmfc\b)", \
                       ppd_make_and_model, re.I):
         make = "Brother"
         model = ppd_make_and_model
-    elif re.search ("^(docuprint|docupage|phaser|workcentre|homecentre)", \
+    elif re.search ("^\s*(docuprint|docupage|phaser|workcentre|homecentre)", \
                       ppd_make_and_model, re.I):
         make = "Xerox"
         model = ppd_make_and_model
-    elif re.search ("^(optra|(color\s*|)jetprinter)", \
+    elif re.search ("^\s*(optra|(color\s*|)jetprinter)", \
                       ppd_make_and_model, re.I):
         make = "Lexmark"
         model = ppd_make_and_model
-    elif re.search ("^(magicolor|pageworks|pagepro)", \
+    elif re.search ("^\s*(magicolor|pageworks|pagepro)", \
                       ppd_make_and_model, re.I):
         make = "KONICA MINOLTA"
         model = ppd_make_and_model
-    elif re.search ("^(aficio)", \
+    elif re.search ("^\s*(aficio)", \
                       ppd_make_and_model, re.I):
         make = "Ricoh"
         model = ppd_make_and_model
-    elif re.search ("^(varioprint)", \
+    elif re.search ("^\s*(varioprint)", \
                       ppd_make_and_model, re.I):
         make = "Oce"
         model = ppd_make_and_model
-    elif re.search ("^(okipage|microline)", \
+    elif re.search ("^\s*(okipage|microline)", \
                       ppd_make_and_model, re.I):
         make = "Oki"
         model = ppd_make_and_model
-    elif re.search ("^(konica[\s_-]*minolta)", \
+    elif re.search ("^\s*(konica[\s_-]*minolta)", \
                       ppd_make_and_model, re.I):
         make = "KONICA MINOLTA"
         model = ppd_make_and_model
         model = re.sub ("(?i)KONICA[\s_-]*MINOLTA\s*", "", model, 1)
+    elif re.search("turboprint", ppd_make_and_model, re.I):
+        # Support for Turboprint PPDs
+        t = ppd_make_and_model.find (" TurboPrint")
+        if t != -1:
+            ppd_make_and_model = ppd_make_and_model[:t]
+        try:
+            make, model = ppd_make_and_model.split("_", 1)
+        except:
+            make = ppd_make_and_model
+            model = ''
+        make = re.sub (r"(?<=[a-z])(?=[0-9])", " ", make)
+        make = re.sub (r"(?<=[a-z])(?=[A-Z])", " ", make)
+        model = re.sub (r"(?<=[a-z])(?=[0-9])", " ", model)
+        model = re.sub (r"(?<=[a-z])(?=[A-Z])", " ", model)
+        model = re.sub (r" Jet", "Jet", model)
+        model = re.sub (r"Photo Smart", "PhotoSmart", model)
     else:
         try:
             make, model = ppd_make_and_model.split(" ", 1)
@@ -99,11 +116,6 @@ def ppdMakeModelSplit (ppd_make_and_model):
             make = ppd_make_and_model
             model = ''
 
-    def strip_suffix (model, suffix):
-        if model.endswith (suffix):
-            return model[:-len(suffix)]
-        return model
-
     # Model names do not contain a comma, truncate all from the
     # comma on
     c = model.find (",")
@@ -120,6 +132,10 @@ def ppdMakeModelSplit (ppd_make_and_model):
         # Truncate at that point.
         model = model[:v]
 
+    h = model.find (" hpijs")
+    if h != -1:
+        model = model[:h]
+
     f = model.find (" Foomatic/")
     if f != -1:
         model = model[:f]
@@ -142,9 +158,16 @@ def ppdMakeModelSplit (ppd_make_and_model):
 
     make = re.sub (r"(?i)KONICA[\s_-]*MINOLTA", "KONICA MINOLTA", make, 1)
     make = re.sub (r"(?i)HEWLETT[\s_-]PACKARD", "HP", make, 1)
+    make = re.sub (r"(?i)Lexmark\s*International", "Lexmark", make, 1)
     model = re.sub (r"(?i)\s*\(recommended\)", "", model)
     model = re.sub (r"(?i)\s*-\s*PostScript\b", "", model)
     model = re.sub (r"(?i)\s*\bseries\b", "", model)
+    if make.lower () == "hp":
+        model = re.sub (r"(?i)^dj\b", "DeskJet", model)
+        model = re.sub (r"(?i)^LJ\b", "LaserJet", model)
+        model = re.sub (r"(?i)^OJ\b", "OfficeJet", model)
+        model = re.sub (r"(?i)^Color\s*LJ\b", "Color LaserJet", model)
+        model = re.sub (r"(?i)^PS\b", "PhotoSmart", model)
     model = re.sub (r"(?i)\s*\bPS[123]?\b", "", model)
     model = re.sub (r"(?i)\s*\bPXL", "", model)
     model = re.sub (r"(?i)[\s_-]+BT\b", "", model)
@@ -179,9 +202,12 @@ DRIVER_TYPE_FOOMATIC_GUTENPRINT = 60
 DRIVER_TYPE_FOOMATIC = 70
 DRIVER_TYPE_CUPS = 80
 DRIVER_TYPE_FOOMATIC_GENERIC = 90
+DRIVER_TYPE_3RD_PARTY_NONFREE = 95
 DRIVER_DOES_NOT_WORK = 999
 def _getDriverType (ppdname, ppds=None):
     """Decides which of the above types ppdname is."""
+    if ppdname.find ("turboprint") != -1:
+        return DRIVER_TYPE_3RD_PARTY_NONFREE
     if ppdname.find ("gutenprint") != -1:
         if (ppdname.find ("/simple/") != -1 or
             ppdname.find (".sim-") != -1):
@@ -327,7 +353,7 @@ class PPDs:
         particular printer model, given its make and model name.
 
 	@returns: a dict, indexed by ppd-name, of dicts representing
-        PPDs (as given by cups.Connection.getPPDs)"
+        PPDs (as given by cups.Connection.getPPDs)
 	"""
         self._init_makes ()
         try:
@@ -387,29 +413,34 @@ class PPDs:
                 else:
                     return -1
 
-            # Prefer C locale localized PPDs to other languages,
-            # just because we don't know the user's locale.
+            # Prefer C locale localized PPDs to other languages, just
+            # because we don't know the user's locale.  This only
+            # applies to PPDs provided by gutenprint 5.0; in 5.2 the
+            # PPDs are localized properly.
             def is_C_locale (x):
-                while x:
-                    i = x.find ("C")
-                    if i == -1:
-                        return False
-                    lword = False
-                    if i == 0:
-                        lword = True
-                    elif x[i - 1] not in string.letters:
-                        lword = True
+                try:
+                    while x:
+                        i = x.find ("C")
+                        if i == -1:
+                            return False
+                        lword = False
+                        if i == 0:
+                            lword = True
+                        elif x[i - 1] not in string.letters:
+                            lword = True
 
-                    if lword:
-                        rword = False
-                        if i == (len (x) - 1):
-                            rword = True
-                        elif x[i + 1] not in string.letters:
-                            rword = True
-                        if rword:
-                            return True
+                        if lword:
+                            rword = False
+                            if i == (len (x) - 1):
+                                rword = True
+                            elif x[i + 1] not in string.letters:
+                                rword = True
+                            if rword:
+                                return True
                         
-                    x = x[i + 1:]
+                        x = x[i + 1:]
+                except UnicodeDecodeError:
+                    return False
 
             ca = is_C_locale (a)
             cb = is_C_locale (b)
@@ -470,23 +501,40 @@ class PPDs:
 	"""
         _debugprint ("\n%s %s" % (mfg, mdl))
         self._init_ids ()
+
+        # Consider "HP" and "Hewlett-Packard" as equal, as the ID returned
+        # by the CUPS "usb" backend and HPLIP's "hp" backend are different
+        if mfg.lower () == "hewlett-packard":
+            mfg = "HP"
+
+        ppdnamelist = []
         id_matched = False
         try:
             ppdnamelist = self.ids[mfg.lower ()][mdl.lower ()]
             status = self.STATUS_SUCCESS
             id_matched = True
         except KeyError:
-            if uri and (uri.startswith ("hp:") or uri.startswith ("hpfax:")):
-                # The HPLIP backends make up incorrect IDs.
-                if mfg == "HP":
-                    try:
-                        ppdnamelist = self.ids['hewlett-packard'][mdl.lower ()]
-                        status = self.STATUS_SUCCESS
-                        id_matched = True
-                    except KeyError:
-                        pass
-            if not id_matched:
-                ppdnamelist = None
+            pass
+
+        mfgl = mfg.lower ()
+        mdll = mdl.lower ()
+
+        # The HPLIP PPDs have incorrect IDs
+        for mf in ["hp", "apollo"]:
+            if mfgl == mf:
+                try:
+                    ppdnamelist += self.ids[mf][mdll]
+                    status = self.STATUS_SUCCESS
+                    id_matched = True
+                except KeyError:
+                    pass
+
+                try:
+                    ppdnamelist += self.ids[mf][mdll.replace (" ", "_")]
+                    status = self.STATUS_SUCCESS
+                    id_matched = True
+                except KeyError:
+                    pass
 
         _debugprint ("Trying make/model names")
         mfgl = mfg.lower ()
@@ -503,6 +551,9 @@ class PPDs:
                 mfgl = "hp"
             elif mfgl == "lexmark international":
                 mfgl = "lexmark"
+            elif mfgl == "":
+                (tmp1, tmp2) = ppdMakeModelSplit (mdl)
+                mfgl = tmp1.lower ()
 
         # Remove manufacturer name from model field
         ppdnamelist2 = None
@@ -512,16 +563,22 @@ class PPDs:
             mdl = mdl[16:]
         if mdl.startswith ('HP '):
             mdl = mdl[3:]
-        if mdls and mdls.has_key (mdl):
-            ppdnamelist2 = mdls[mdl].keys ()
-            status = self.STATUS_SUCCESS
-        else:
-            # Make use of the model name clean-up in the ppdMakeModelSplit ()
-            # function
-            (mfg2, mdl2) = ppdMakeModelSplit (mfg + " " + mdl)
-            if mdls and mdls.has_key (mdl2):
-                ppdnamelist2 = mdls[mdl2].keys ()
-                status = self.STATUS_SUCCESS
+        if mdls:
+            for (model, ppdnames) in mdls.iteritems ():
+                if model.lower () == mdll:
+                    ppdnamelist2 = ppdnames.keys ()
+                    status = self.STATUS_SUCCESS
+                    break
+            if not ppdnamelist2:
+                # Make use of the model name clean-up in the
+                # ppdMakeModelSplit () function
+                (mfg2, mdl2) = ppdMakeModelSplit (mfg + " " + mdl)
+                mdl2l = mdl2.lower ()
+                for (model, ppdnames) in mdls.iteritems ():
+                    if model.lower () == mdl2l:
+                        ppdnamelist2 = ppdnames.keys ()
+                        status = self.STATUS_SUCCESS
+                        break
       
         if ppdnamelist:
             if ppdnamelist2:
@@ -545,36 +602,39 @@ class PPDs:
                 ppdnamelist = generic
 
         if not ppdnamelist:
-            _debugprint ("Text-only fallback")
             status = self.STATUS_NO_DRIVER
-            ppdnamelist = ["textonly.ppd"]
-            tppdfound = 0
-            for ppdpath in self.ppds.keys ():
-                if ppdpath.endswith (ppdnamelist[0]):
-                    tppdfound = 1
-                    ppdnamelist = [ppdpath]
-                    break
-            if tppdfound == 0:
-                _debugprint ("No text-only driver?!  Using postscript.ppd")
-                ppdnamelist = ["postscript.ppd"]
-                psppdfound = 0
+            fallbacks = ["textonly.ppd", "postscript.ppd"]
+            found = False
+            for fallback in fallbacks:
+                _debugprint ("'%s' fallback" % fallback)
+                fallbackgz = fallback + ".gz"
                 for ppdpath in self.ppds.keys ():
-                    if ppdpath.endswith (ppdnamelist[0]):
-                        psppdfound = 1
+                    if (ppdpath.endswith (fallback) or
+                        ppdpath.endswith (fallbackgz)):
                         ppdnamelist = [ppdpath]
+                        found = True
                         break
-                if psppdfound == 0:
-                    _debugprint ("No postscript.ppd; choosing any")
-                    ppdnamelist = [self.ppds.keys ()[0]]
+
+                if found:
+                    break
+
+                _debugprint ("Fallback '%s' not available" % fallback)
+
+            if not found:
+                _debugprint ("No fallback available; choosing any")
+                ppdnamelist = [self.ppds.keys ()[0]]
 
         if id_matched:
             _debugprint ("Checking DES field")
             inexact = set()
             if description:
                 for ppdname in ppdnamelist:
+                    if ppdname.find ("hpijs"):
+                        continue
                     ppddict = self.ppds[ppdname]
                     id = ppddict['ppd-device-id']
-                    if not id: continue
+                    if not id:
+                        continue
                     # Fetch description field.
                     id_dict = parseDeviceID (id)
                     if id_dict["DES"] != description:
@@ -591,7 +651,7 @@ class PPDs:
         # take the first.
         ppdnamelist = self.orderPPDNamesByPreference (ppdnamelist,
                                                       downloadedfiles)
-        _debugprint (str (ppdnamelist))
+        _debugprint ("Found PPDs: %s" % str (ppdnamelist))
 
         if not id_matched:
             sanitised_uri = re.sub (pattern="//[^@]*@/?", repl="//",
@@ -689,43 +749,45 @@ class PPDs:
                         digits_start = i
                     digits_end = i
                     digits += 1
+                elif digits_start != -1:
+                    break
             digits_end += 1
             modelnumber = 0
             if digits > 0:
                 modelnumber = int (modelid[digits_start:digits_end])
-            modelpattern = (modelid[:digits_start] + "%d" +
-                            modelid[digits_end:])
-            _debugprint ("Searching for model ID '%s', '%s' %% %d" %
-                        (modelid, modelpattern, modelnumber))
-            ignore_digits = 0
-            best_mdl = None
-            found = False
-            while ignore_digits < digits:
-                div = pow (10, ignore_digits)
-                modelid = modelpattern % ((modelnumber / div) * div)
-                _debugprint ("Ignoring %d of %d digits, trying %s" %
-                            (ignore_digits, digits, modelid))
+                modelpattern = (modelid[:digits_start] + "%d" +
+                                modelid[digits_end:])
+                _debugprint ("Searching for model ID '%s', '%s' %% %d" %
+                             (modelid, modelpattern, modelnumber))
+                ignore_digits = 0
+                best_mdl = None
+                found = False
+                while ignore_digits < digits:
+                    div = pow (10, ignore_digits)
+                    modelid = modelpattern % ((modelnumber / div) * div)
+                    _debugprint ("Ignoring %d of %d digits, trying %s" %
+                                 (ignore_digits, digits, modelid))
 
-                for (name, ppds) in mdlitems:
-                    for word in name.split (' '):
-                        if word.lower () == modelid:
-                            found = True
+                    for (name, ppds) in mdlitems:
+                        for word in name.split (' '):
+                            if word.lower () == modelid:
+                                found = True
+                                break
+
+                        if found:
+                            best_mdl = ppds.keys ()
                             break
 
                     if found:
-                        best_mdl = ppds.keys ()
+                        break
+
+                    ignore_digits += 1
+                    if digits < 2:
                         break
 
                 if found:
-                    break
-
-                ignore_digits += 1
-                if digits < 2:
-                    break
-
-            if found:
-                ppdnamelist = best_mdl
-                status = self.STATUS_MODEL_MISMATCH
+                    ppdnamelist = best_mdl
+                    status = self.STATUS_MODEL_MISMATCH
 
         return (status, ppdnamelist)
 
@@ -817,6 +879,11 @@ class PPDs:
             lmfg = id_dict['MFG'].lower ()
             lmdl = id_dict['MDL'].lower ()
 
+            # Consider "HP" and "Hewlett-Packard" as equal, as the ID returned
+            # by the CUPS "usb" backend and HPLIP's "hp" backend are different
+            if lmfg == "hewlett-packard":
+                lmfg = "hp"
+
             bad = False
             if len (lmfg) == 0:
                 _debugprint ("Missing MFG field for %s" % ppdname)
