@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
-## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
-## Copyright (C) 2007, 2008 Red Hat, Inc.
+## Copyright (C) 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2007, 2008, 2009 Red Hat, Inc.
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 ## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import cups
+cups.require("1.9.42")
 import dbus
 import dbus.glib
 import gobject
@@ -54,9 +55,6 @@ def collect_printer_state_reasons (connection):
 
     for name, printer in printers.iteritems ():
         reasons = printer["printer-state-reasons"]
-        if type (reasons) != list:
-            # Work around a bug that was fixed in pycups-1.9.20.
-            reasons = [reasons]
         for reason in reasons:
             if reason == "none":
                 break
@@ -129,6 +127,8 @@ class Monitor:
         self.jobs = {}
         self.printer_state_reasons = {}
         self.printers = set()
+        self.process_pending_events = True
+        self.fetch_jobs_timer = None
 
         if host:
             cups.setServer (host)
@@ -136,6 +136,7 @@ class Monitor:
             cups.setPort (port)
         if encryption:
             cups.setEncryption (encryption)
+        self.user = cups.getUser ()
         self.host = cups.getServer ()
         self.port = cups.getPort ()
         self.encryption = cups.getEncryption ()
@@ -146,14 +147,20 @@ class Monitor:
         self.still_connecting = set()
         self.connecting_to_device = {}
         self.received_any_dbus_signals = False
+        self.update_timer = None
 
         if bus == None:
-            bus = dbus.SystemBus ()
+            try:
+                bus = dbus.SystemBus ()
+            except dbus.exceptions.DBusException:
+                # System bus not running.
+                pass
 
-        bus.add_signal_receiver (self.handle_dbus_signal,
-                                 path=self.DBUS_PATH,
-                                 dbus_interface=self.DBUS_IFACE)
-        self.bus = bus
+        if bus != None:
+            bus.add_signal_receiver (self.handle_dbus_signal,
+                                     path=self.DBUS_PATH,
+                                     dbus_interface=self.DBUS_IFACE)
+            self.bus = bus
 
         self.sub_id = -1
         self.refresh ()
@@ -163,31 +170,44 @@ class Monitor:
 
     def cleanup (self):
         if self.sub_id != -1:
+            user = cups.getUser ()
             try:
-                try:
-                    c = cups.Connection (host=self.host,
-                                         port=self.port,
-                                         encryption=self.encryption)
-                except TypeError:
-                    # Parameters to Connection() requires pycups >= 1.9.40.
-                    cups.setServer (self.host)
-                    cups.setPort (self.port)
-                    cups.setEncryption (self.encryption)
-                    c = cups.Connection ()
-
+                cups.setUser (self.user)
+                c = cups.Connection (host=self.host,
+                                     port=self.port,
+                                     encryption=self.encryption)
                 c.cancelSubscription (self.sub_id)
                 debugprint ("Canceled subscription %d" % self.sub_id)
             except:
                 pass
+            cups.setUser (user)
 
-        self.bus.remove_signal_receiver (self.handle_dbus_signal,
-                                         path=self.DBUS_PATH,
-                                         dbus_interface=self.DBUS_IFACE)
+        if self.bus != None:
+            self.bus.remove_signal_receiver (self.handle_dbus_signal,
+                                             path=self.DBUS_PATH,
+                                             dbus_interface=self.DBUS_IFACE)
+
+        timers = self.connecting_timers.values ()
+        for timer in [self.update_timer, self.fetch_jobs_timer]:
+            if timer:
+                timers.append (timer)
+        for timer in timers:
+            gobject.source_remove (timer)
 
         self.watcher.monitor_exited (self)
 
+    def set_process_pending (self, whether):
+        self.process_pending_events = whether
+
     def check_still_connecting(self, printer):
         """Timer callback to check on connecting-to-device reasons."""
+        if not self.process_pending_events:
+            # Defer the timer by setting a new one.
+            timer = gobject.timeout_add (200, self.check_still_connecting,
+                                         printer)
+            self.connecting_timers[printer] = timer
+            return False
+
         del self.connecting_timers[printer]
         debugprint ("Still-connecting timer fired for `%s'" % printer)
         (printer_jobs, my_printers) = self.sort_jobs_by_printer ()
@@ -226,8 +246,9 @@ class Monitor:
                     debugprint ("Connecting time: %d" % (time_now - t))
                     if time_now - t >= CONNECTING_TIMEOUT:
                         if have_processing_job:
-                            self.still_connecting.add (printer)
-                            self.watcher.still_connecting (self, reason)
+                            if printer not in self.still_connecting:
+                                self.still_connecting.add (printer)
+                                self.watcher.still_connecting (self, reason)
                             if self.connecting_timers.has_key (printer):
                                 gobject.source_remove (self.connecting_timers
                                                        [printer])
@@ -308,18 +329,22 @@ class Monitor:
                 self.watcher.state_reason_removed (self, reason)
 
     def get_notifications(self):
+        if not self.process_pending_events:
+            # Defer the timer callback.
+            if self.update_timer:
+                gobject.source_remove (self.update_timer)
+
+            self.update_timer = gobject.timeout_add (200,
+                                                     self.get_notifications)
+            return False
+
         debugprint ("get_notifications")
+        user = cups.getUser ()
         try:
-            try:
-                c = cups.Connection (host=self.host,
-                                     port=self.port,
-                                     encryption=self.encryption)
-            except TypeError:
-                # Parameters to Connection() requires pycups >= 1.9.40.
-                cups.setServer (self.host)
-                cups.setPort (self.port)
-                cups.setEncryption (self.encryption)
-                c = cups.Connection ()
+            cups.setUser (self.user)
+            c = cups.Connection (host=self.host,
+                                 port=self.port,
+                                 encryption=self.encryption)
 
             try:
                 try:
@@ -328,6 +353,7 @@ class Monitor:
                 except AttributeError:
                     notifications = c.getNotifications ([self.sub_id])
             except cups.IPPError, (e, m):
+                cups.setUser (user)
                 if e == cups.IPP_NOT_FOUND:
                     # Subscription lease has expired.
                     self.sub_id = -1
@@ -337,19 +363,15 @@ class Monitor:
                 self.watcher.cups_ipp_error (self, e, m)
                 return True
         except RuntimeError:
+            cups.setUser (user)
             self.watcher.cups_connection_error (self)
             return True
 
+        cups.setUser (user)
         deferred_calls = []
         jobs = self.jobs.copy ()
         for event in notifications['events']:
             seq = event['notify-sequence-number']
-            try:
-                if seq <= self.sub_seq:
-                    # Work around a bug in pycups < 1.9.34
-                    continue
-            except AttributeError:
-                pass
             self.sub_seq = seq
             nse = event['notify-subscribed-event']
             debugprint ("%d %s %s" % (seq, nse, event['notify-text']))
@@ -380,10 +402,6 @@ class Monitor:
                                             (self, name)))
                 elif name in self.printers:
                     printer_state_reasons = event['printer-state-reasons']
-                    if type (printer_state_reasons) != list:
-                        # Work around a bug in pycups < 1.9.36
-                        printer_state_reasons = [printer_state_reasons]
-
                     reasons = []
                     for reason in printer_state_reasons:
                         if reason == "none":
@@ -449,16 +467,20 @@ class Monitor:
             deferred_calls.append ((self.watcher.job_event,
                                    (self, jobid, nse, event, job.copy ())))
 
-        self.update (jobs)
+        self.set_process_pending (False)
+        self.update_jobs (jobs)
         self.jobs = jobs
 
         for (fn, args) in deferred_calls:
             fn (*args)
+        self.set_process_pending (True)
 
         # Update again when we're told to.  If we're getting CUPS
         # D-Bus signals, however, rely on those instead.
         if not self.received_any_dbus_signals:
-            gobject.source_remove (self.update_timer)
+            if self.update_timer:
+                gobject.source_remove (self.update_timer)
+
             interval = 1000 * notifications['notify-get-interval']
             self.update_timer = gobject.timeout_add (interval,
                                                      self.get_notifications)
@@ -471,19 +493,15 @@ class Monitor:
         if which_jobs != None:
             self.which_jobs = which_jobs
 
+        user = cups.getUser ()
         try:
-            try:
-                c = cups.Connection (host=self.host,
-                                     port=self.port,
-                                     encryption=self.encryption)
-            except TypeError:
-                # Parameters to Connection() requires pycups >= 1.9.40.
-                cups.setServer (self.host)
-                cups.setPort (self.port)
-                cups.setEncryption (self.encryption)
-                c = cups.Connection ()
+            cups.setUser (self.user)
+            c = cups.Connection (host=self.host,
+                                 port=self.port,
+                                 encryption=self.encryption)
         except RuntimeError:
             self.watcher.cups_connection_error (self)
+            cups.setUser (user)
             return
 
         if self.sub_id != -1:
@@ -492,7 +510,9 @@ class Monitor:
             except cups.IPPError, (e, m):
                 self.watcher.cups_ipp_error (self, e, m)
 
-            gobject.source_remove (self.update_timer)
+            if self.update_timer:
+                gobject.source_remove (self.update_timer)
+
             debugprint ("Canceled subscription %d" % self.sub_id)
 
         try:
@@ -514,6 +534,8 @@ class Monitor:
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
 
+        cups.setUser (user)
+
         self.update_timer = gobject.timeout_add (MIN_REFRESH_INTERVAL * 1000,
                                                  self.get_notifications)
         debugprint ("Created subscription %d" % self.sub_id)
@@ -529,21 +551,17 @@ class Monitor:
                 jobs = filtered
 
             self.fetch_first_job_id = 1
-            gobject.timeout_add (5, self.fetch_jobs, refresh_all)
+            if self.fetch_jobs_timer:
+                gobject.source_remove (self.fetch_jobs_timer)
+            self.fetch_jobs_timer = gobject.timeout_add (5, self.fetch_jobs,
+                                                         refresh_all)
         else:
             jobs = {}
 
         try:
             self.printer_state_reasons = collect_printer_state_reasons (c)
-            dests = c.getDests ()
-            printers = set()
-            for (printer, instance) in dests.keys ():
-                if printer == None:
-                    continue
-                if instance != None:
-                    continue
-                printers.add (printer)
-            self.printers = printers
+            dests = c.getPrinters ()
+            self.printers = set(dests.keys ())
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
             return
@@ -559,27 +577,29 @@ class Monitor:
                 if printer not in self.specific_dests:
                     del jobs[jobid]
 
+        self.set_process_pending (False)
         self.watcher.current_printers_and_jobs (self, self.printers.copy (),
                                                 jobs.copy ())
-        self.update (jobs)
-
+        self.update_jobs (jobs)
         self.jobs = jobs
+        self.set_process_pending (True)
         return False
 
     def fetch_jobs (self, refresh_all):
+        if not self.process_pending_events:
+            # Skip this call.  We'll get called again soon.
+            return True
+
+        user = cups.getUser ()
         try:
-            try:
-                c = cups.Connection (host=self.host,
-                                     port=self.port,
-                                     encryption=self.encryption)
-            except TypeError:
-                # Parameters to Connection() requires pycups >= 1.9.40.
-                cups.setServer (self.host)
-                cups.setPort (self.port)
-                cups.setEncryption (self.encryption)
-                c = cups.Connection ()
+            cups.setUser (self.user)
+            c = cups.Connection (host=self.host,
+                                 port=self.port,
+                                 encryption=self.encryption)
         except RuntimeError:
             self.watcher.cups_connection_error (self)
+            self.fetch_jobs_timer = None
+            cups.setUser (user)
             return False
 
         limit = 1
@@ -588,15 +608,13 @@ class Monitor:
                                  my_jobs=self.my_jobs,
                                  first_job_id=self.fetch_first_job_id,
                                  limit=limit)
-        except TypeError:
-            # limit requires pycups >= 1.9.42
-            fetched = c.getJobs (which_jobs=self.which_jobs,
-                                 my_jobs=self.my_jobs)
-            limit = len (fetched) + 1
         except cups.IPPError, (e, m):
             self.watcher.cups_ipp_error (self, e, m)
+            self.fetch_jobs_timer = None
+            cups.setUser (user)
             return False
 
+        cups.setUser (user)
         got = len (fetched)
         debugprint ("Got %s jobs, asked for %s" % (got, limit))
 
@@ -611,6 +629,13 @@ class Monitor:
         for jobid in xrange (self.fetch_first_job_id, last_jobid + 1):
             try:
                 job = fetched[jobid]
+                if self.specific_dests != None:
+                    uri = job.get('job-printer-uri', '/')
+                    i = uri.rfind ('/')
+                    printer = uri[i + 1:]
+                    if printer not in self.specific_dests:
+                        raise KeyError
+
                 if jobs.has_key (jobid):
                     fn = self.watcher.job_event
                 else:
@@ -626,7 +651,21 @@ class Monitor:
                     deferred_calls.append ((self.watcher.job_removed,
                                             (self, jobid, '', {})))
 
-        self.update (jobs)
+        jobids = jobs.keys ()
+        jobids.sort ()
+        if got < limit:
+            trim = False
+            for i in range (len (jobids)):
+                jobid = jobids[i]
+                if not trim and jobid > last_jobid:
+                    trim = True
+            
+                if trim:
+                    del jobs[jobid]
+                    deferred_calls.append ((self.watcher.job_removed,
+                                            (self, jobid, '', {})))
+
+        self.update_jobs (jobs)
         self.jobs = jobs
 
         for (fn, args) in deferred_calls:
@@ -634,6 +673,7 @@ class Monitor:
 
         if got < limit:
             # That's all.  Don't run this timer again.
+            self.fetch_jobs_timer = None
             return False
 
         # Remember where we got up to and run this timer again.
@@ -667,14 +707,19 @@ class Monitor:
 
         return (printer_jobs, my_printers)
 
-    def update(self, jobs):
-        debugprint ("update")
+    def update_jobs(self, jobs):
+        debugprint ("update_jobs")
         (printer_jobs, my_printers) = self.sort_jobs_by_printer (jobs)
         self.check_state_reasons (my_printers, printer_jobs)
 
-    def handle_dbus_signal(self, *args):
-        gobject.source_remove (self.update_timer)
+    def update(self):
+        if self.update_timer:
+            gobject.source_remove (self.update_timer)
+
         self.update_timer = gobject.timeout_add (200, self.get_notifications)
+
+    def handle_dbus_signal(self, *args):
+        self.update ()
         if not self.received_any_dbus_signals:
             self.received_any_dbus_signals = True
 
